package tofly.opc_gateway.config;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Created by huangdecai on 2023/4/19.
 * OPC server configuration
 */
@Configuration
@ConfigurationProperties(prefix = "opc.gateway.servers")
public class OpcServerProperties {

    private Map<String, ServerConfiguration> mappings = new LinkedHashMap<>();

    public OpcServerProperties() {
        super();
    }

    public Map<String, ServerConfiguration> getMappings() {
        return mappings;
    }

    public void setMappings(Map<String, ServerConfiguration> mappings) {
        this.mappings = mappings;
    }

    public static class ServerConfiguration {
        private final static String DEFAULT_HOST = "127.0.0.1";
        private final static int DEFAULT_PERIOD = 1000;
        private String host;

        private String progId;

        private String clsid;

        private int period;

        private Map<String, DeviceConfiguration> deviceMappings = new LinkedHashMap<>();

        public ServerConfiguration() {
            this.host = DEFAULT_HOST;
            this.period = DEFAULT_PERIOD;
        }

        public String getHost() {
            return host;
        }

        public void setHost(String host) {
            this.host = host;
        }

        public String getProgId() {
            return progId;
        }

        public void setProgId(String progId) {
            this.progId = progId;
        }

        public String getClsid() {
            return clsid;
        }

        public void setClsid(String clsid) {
            this.clsid = clsid;
        }

        public int getPeriod() {
            return period;
        }

        public void setPeriod(int period) {
            this.period = period;
        }

        public Map<String, DeviceConfiguration> getDeviceMappings() {
            return deviceMappings;
        }

        public void setDeviceMappings(Map<String, DeviceConfiguration> deviceMappings) {
            this.deviceMappings = deviceMappings;
        }
    }

    public static class DeviceConfiguration {
        private final static String DEFAULT_TABLE = "tsdata";
        private String table;


        private Map<String, FieldConfiguration> fieldMappings = new LinkedHashMap<>();

        public DeviceConfiguration() {
            this.table = DEFAULT_TABLE;
        }

        public String getTable() {
            return table;
        }

        public void setTable(String table) {
            this.table = table;
        }

        public Map<String,FieldConfiguration> getFieldMappings() {
            return fieldMappings;
        }

        public void setFieldMappings(Map<String, FieldConfiguration> fieldMappings) {
            this.fieldMappings = fieldMappings;
        }
    }

    public static class FieldConfiguration {

        private String label;
        private String name;
        private boolean intType;

        //放大倍数
        private double magnification = 1;

        //基数
        private double cardinality = 0;

        //保留小数位数
        private Integer digits;

        public boolean isIntType() {
            return intType;
        }

        public void setIntType(boolean intType) {
            this.intType = intType;
        }

        public String getLabel() {
            return label;
        }

        public void setLabel(String label) {
            this.label = label;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public double getMagnification() {
            return magnification;
        }

        public void setMagnification(double magnification) {
            this.magnification = magnification;
        }

        public Integer getDigits() {
            return digits;
        }

        public void setDigits(Integer digits) {
            this.digits = digits;
        }

        public double getCardinality() {
            return cardinality;
        }

        public void setCardinality(double cardinality) {
            this.cardinality = cardinality;
        }
    }





}
